#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var commander_1 = require("commander");
var nice_color_palettes_1 = __importDefault(require("nice-color-palettes"));
var chalk_1 = __importDefault(require("chalk"));
var utils_1 = require("./utils");
commander_1.program
    .version("1.1.0")
    .description("Displays an animated simplex noise field in your terminal")
    .option("-a, --angle", "outputs noise field as angles")
    .option("-b, --block", "outputs noise field as colour blocks")
    .option("-n, --number", "shows the numeric value of the noise field at each point")
    .parse(process.argv);
var clearScrollback = function () {
    return process.stdout.write("\u001b[H\u001b[2J\u001b[3J");
};
var createGrid = function () {
    var pointPositions = [];
    var columns = process.stdout.columns;
    var rows = process.stdout.rows;
    for (var x = 0; x < rows; x++) {
        for (var y = 0; y < columns; y++) {
            pointPositions.push([x, y]);
        }
    }
    return pointPositions;
};
var drawGridTimeout;
var startDrawLoop = function (pointPositions) {
    var nColours = 5;
    var randomPalette = utils_1.shuffle(utils_1.pickRandom(nice_color_palettes_1.default).slice(0, nColours));
    var color;
    var char = " ";
    var noiseZ = 0;
    var noiseZVel = 0.0000022;
    var zMult;
    var chalkWrite = function (color) {
        return commander_1.program.block ? chalk_1.default.bgHex(color) : chalk_1.default.hex(color);
    };
    var drawBlock = function (z) {
        var nChars = 5;
        zMult = Math.abs(Math.floor(z * nChars));
        var colourMap = Math.floor(utils_1.mapRange(zMult, 0, nChars - 1, 0, nColours - 1));
        color = randomPalette[colourMap];
        return [color];
    };
    var drawAngle = function (z) {
        var nChars = 4;
        zMult = Math.abs(Math.floor(z * nChars));
        var colourMap = Math.floor(utils_1.mapRange(zMult, 0, nChars - 1, 0, nColours - 1));
        color = randomPalette[colourMap];
        if (zMult === 0)
            char = "|";
        if (zMult === 1)
            char = "/";
        if (zMult === 2)
            char = "—";
        if (zMult === 3)
            char = "\\";
        return [color, char];
    };
    var drawAscii = function (z) {
        var nChars = 9;
        zMult = Math.abs(Math.floor(z * nChars));
        var colourMap = Math.floor(utils_1.mapRange(zMult, 4, nChars - 1, 0, nColours - 1));
        color = randomPalette[colourMap];
        if ([0, 1, 2, 3].includes(zMult))
            char = " ";
        if (zMult === 4)
            char = ".";
        if (zMult === 5)
            char = "-";
        if (zMult === 6)
            char = "o";
        if (zMult === 7)
            char = "+";
        if (zMult === 8)
            char = "x";
        if (zMult === 9)
            char = "X";
        return [color, char];
    };
    drawGridTimeout = setInterval(function () {
        clearScrollback();
        pointPositions.forEach(function (position) {
            var _a, _b;
            var y = position[0], x = position[1];
            var z = utils_1.noise3D(x, y, noiseZ);
            if (commander_1.program.block) {
                color = drawBlock(z)[0];
            }
            else if (commander_1.program.angle) {
                _a = drawAngle(z), color = _a[0], char = _a[1];
            }
            else {
                _b = drawAscii(z), color = _b[0], char = _b[1];
            }
            if (commander_1.program.number)
                char = zMult;
            process.stdout.write(chalkWrite(color)(char));
            noiseZ += noiseZVel;
        });
    }, 80);
};
var init = function () {
    var grid = createGrid();
    startDrawLoop(grid);
};
init();
var initTimeout;
process.stdout.on("resize", function () {
    clearScrollback();
    clearInterval(drawGridTimeout);
    clearTimeout(initTimeout);
    initTimeout = setTimeout(function () { return init(); }, 200);
});
